@@ -38,7 +38,7 @@ from io import BytesIO
 
 
 API_BASE_URL = 'http://api.acoustid.org/v2/'
-DEFAULT_META = 'recordings'
+DEFAULT_META = ['recordings']
 REQUEST_INTERVAL = 0.33  # 3 requests/second.
 MAX_AUDIO_LENGTH = 120  # Seconds.
 FPCALC_COMMAND = 'fpcalc'
@@ -112,10 +112,12 @@ def _get_submit_url():
     """Get the URL of the submission API endpoint."""
     return API_BASE_URL + 'submit'
 
+
 def _get_submission_status_url():
     """Get the URL of the submission status API endpoint."""
     return API_BASE_URL + 'submission_status'
 
+
 # Compressed HTTP request bodies.
 
 def _compress(data):
@@ -166,22 +168,32 @@ class _rate_limit(object):  # noqa: N801
 
 
 @_rate_limit
-def _api_request(url, params):
+def _api_request(url, params, timeout=None):
     """Makes a POST request for the URL with the given form parameters,
     which are encoded as compressed form data, and returns a parsed JSON
     response. May raise a WebServiceError if the request fails.
+    If the specified timeout passes, then raises a TimeoutError.
     """
     headers = {
         'Accept-Encoding': 'gzip',
         "Content-Type": "application/x-www-form-urlencoded"
     }
 
-    session = requests.Session()
-    session.mount('http://', CompressedHTTPAdapter())
-    try:
-        response = session.post(url, data=params, headers=headers)
-    except requests.exceptions.RequestException as exc:
-        raise WebServiceError("HTTP request failed: {0}".format(exc))
+    with requests.Session() as session:
+        session.mount('http://', CompressedHTTPAdapter())
+        try:
+            if isinstance(params['meta'], list):
+                params['meta'] = ' '.join(params['meta'])
+            response = session.post(url,
+                                    data=params,
+                                    headers=headers,
+                                    timeout=timeout)
+        except requests.exceptions.RequestException as exc:
+            raise WebServiceError("HTTP request failed: {0}".format(exc))
+        except requests.exceptions.ReadTimeout:
+            raise WebServiceError(
+                "HTTP request timed out ({0}s)".format(timeout)
+            )
 
     try:
         return response.json()
@@ -216,9 +228,12 @@ def fingerprint(samplerate, channels, pcmiter, maxlength=MAX_AUDIO_LENGTH):
         raise FingerprintGenerationError("fingerprint calculation failed")
 
 
-def lookup(apikey, fingerprint, duration, meta=DEFAULT_META):
+def lookup(apikey, fingerprint, duration, meta=DEFAULT_META, timeout=None):
     """Look up a fingerprint with the Acoustid Web service. Returns the
-    Python object reflecting the response JSON data.
+    Python object reflecting the response JSON data. To get more data
+    back, ``meta`` can be a list of keywords from this list: recordings,
+    recordingids, releases, releaseids, releasegroups, releasegroupids,
+    tracks, compress, usermeta, sources.
     """
     params = {
         'format': 'json',
@@ -227,7 +242,7 @@ def lookup(apikey, fingerprint, duration, meta=DEFAULT_META):
         'fingerprint': fingerprint,
         'meta': meta,
     }
-    return _api_request(_get_lookup_url(), params)
+    return _api_request(_get_lookup_url(), params, timeout)
 
 
 def parse_lookup_result(data):
@@ -314,33 +329,40 @@ def _fingerprint_file_fpcalc(path, maxlength):
     return duration, fp
 
 
-def fingerprint_file(path, maxlength=MAX_AUDIO_LENGTH):
+def fingerprint_file(path, maxlength=MAX_AUDIO_LENGTH, force_fpcalc=False):
     """Fingerprint a file either using the Chromaprint dynamic library
-    or the fpcalc command-line tool, whichever is available. Returns the
-    duration and the fingerprint.
+    or the fpcalc command-line tool, whichever is available (unless
+    ``force_fpcalc`` is specified). Returns the duration and the
+    fingerprint.
     """
     path = os.path.abspath(os.path.expanduser(path))
-    if have_audioread and have_chromaprint:
+    if have_audioread and have_chromaprint and not force_fpcalc:
         return _fingerprint_file_audioread(path, maxlength)
     else:
         return _fingerprint_file_fpcalc(path, maxlength)
 
 
-def match(apikey, path, meta=DEFAULT_META, parse=True):
+def match(apikey, path, meta=DEFAULT_META, parse=True, force_fpcalc=False,
+          timeout=None):
     """Look up the metadata for an audio file. If ``parse`` is true,
     then ``parse_lookup_result`` is used to return an iterator over
     small tuple of relevant information; otherwise, the full parsed JSON
-    response is returned.
+    response is returned. Fingerprinting uses either the Chromaprint
+    library or the fpcalc command-line tool; if ``force_fpcalc`` is
+    true, only the latter will be used. To get more data back, ``meta``
+    can be a list of keywords from this list: recordings, recordingids,
+    releases, releaseids, releasegroups, releasegroupids, tracks,
+    compress, usermeta, sources.
     """
-    duration, fp = fingerprint_file(path)
-    response = lookup(apikey, fp, duration, meta)
+    duration, fp = fingerprint_file(path, force_fpcalc=force_fpcalc)
+    response = lookup(apikey, fp, duration, meta, timeout)
     if parse:
         return parse_lookup_result(response)
     else:
         return response
 
 
-def submit(apikey, userkey, data):
+def submit(apikey, userkey, data, timeout=None):
     """Submit a fingerprint to the acoustid server. The ``apikey`` and
     ``userkey`` parameters are API keys for the application and the
     submitting user, respectively.
@@ -378,7 +400,7 @@ def submit(apikey, userkey, data):
         for k, v in d.items():
             args["%s.%s" % (k, i)] = v
 
-    response = _api_request(_get_submit_url(), args)
+    response = _api_request(_get_submit_url(), args, timeout)
     if response.get('status') != 'ok':
         try:
             code = response['error']['code']
@@ -388,7 +410,8 @@ def submit(apikey, userkey, data):
         raise WebServiceError("error {0}: {1}".format(code, message))
     return response
 
-def get_submission_status(apikey, submission_id):
+
+def get_submission_status(apikey, submission_id, timeout=None):
     """Get the status of a submission to the acoustid server.
     ``submission_id`` is the id of a fingerprint submission, as returned
     in the response object of a call to the ``submit`` endpoint.
@@ -398,4 +421,4 @@ def get_submission_status(apikey, submission_id):
         'client': apikey,
         'id': submission_id,
     }
-    return _api_request(_get_submission_status_url(), params)
+    return _api_request(_get_submission_status_url(), params, timeout)
